<?php
/**
 * Phire CMS
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.TXT.
 * It is also available through the world-wide-web at this URL:
 * http://www.phirecms.org/LICENSE.TXT
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@phirecms.org so we can send you a copy immediately.
 *
 * @category   Phire
 * @package    Phire_Form
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 */

/**
 * Phire_Form_AddComment
 *
 * @category   Phire
 * @package    Phire_Form
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 * @version    1.1.2
 */

class Phire_Form_AddComment extends Moc10_Form
{

    /**
     * Initialization method for the add comment form.
     *
     * @param  string $cmt_auth
     * @param  string $cmt_cont
     * @param  string $cap
     * @param  string|int $pid
     * @param  string|int $uid
     * @param  string|int $mid
     * @return void
     */
    public function init($cmt_auth = null, $cmt_cont = null, $cap = null, $hash = null, $pid = 0, $uid = 0, $mid = 0)
    {

        $lang = new Phire_Language();

        $site = new Phire_Table_Sites();
        $site->getSite();

        $this->setAttributes('id', 'commentForm');

        // Set member or user username, if applicable
        if (($uid != 0) && (is_null($cmt_auth))) {
            $usr = new Phire_Table_Users();
            $usr->findById($uid);
            $cmt_auth = $usr->username;
        } else if (($mid != 0) && (is_null($cmt_auth))) {
            $mem = new Phire_Table_Members();
            $mem->findById($mid);
            if ($mem->username != '') {
                $cmt_auth = $mem->username;
            }
        }

        // Create the 'comment_author' form element.
        $comment_author = new Moc10_Form_Element('text', 'comment_author', $cmt_auth);
        $comment_author->setLabel($lang->__('Name:'));
        $comment_author->setAttributes('size', '62');

        // Create the 'comment_content' form element.
        $comment_content = new Moc10_Form_Element_Textarea('comment_content', $cmt_cont);
        $comment_content->setLabel($lang->__('Comment:'));
        $comment_content->setAttributes('cols', '62');
        $comment_content->setAttributes('rows', '5');
        $comment_content->setRequired(true);

        // Create the 'page_id' form element.
        $page_id = new Moc10_Form_Element('hidden', 'page_id', $pid);

        // Create the 'user_id' form element.
        $user_id = new Moc10_Form_Element('hidden', 'user_id', $uid);

        // Create the 'member_id' form element.
        $member_id = new Moc10_Form_Element('hidden', 'member_id', $mid);

        // Create the 'site_id' form element.
        $site_id = new Moc10_Form_Element('hidden', 'site_id', $site->site_id);

        // Create the 'submit' form element.
        $submit = new Moc10_Form_Element('submit', 'cmt_submit', $lang->__('SUBMIT'));
        $submit->setAttributes('class', 'submitBtn');
        $submit->setAttributes('style', 'float: right;');

        // If members are allowed, set member links.
        $mem_links = '';
        if ($site->member_allow == 'Yes') {
            if ($mid != 0) {
                $mem_links .= "<a href=\"/member/logout\">" . $lang->__('Log Out') . "</a>";
            } else {
                if ($site->member_enable_login == 'Yes') {
                    $mem_links .= "<a href=\"/member/login\">" . $lang->__('Login') . "</a> | <a href=\"/member/forgot\">" . $lang->__('Forgot?') . "</a>";
                }
                if ($site->member_enable_registration  == 'Yes') {
                    if ($mem_links == '') {
                        $mem_links .= "<a href=\"/member/register\">" . $lang->__('Register') . "</a>";
                    } else {
                        $mem_links .= " | <a href=\"/member/register\">" . $lang->__('Register') . "</a>";
                    }
                }
            }
        } else {
            $mem_links = '';
        }

        // Define the form template.
        $tmpl = <<<FORMTEMP
            <table id="commentTable" cellpadding="0" cellspacing="0">
                <tr>
                    <td style="text-align: right; vertical-align: top;">
                        [{comment_author_label}]
                    </td>
                    <td>
                        [{comment_author}]
                    </td>
                </tr>
                <tr>
                    <td style="text-align: right; vertical-align: top;">
                        [{comment_content_label}]
                    </td>
                    <td>
                        [{comment_content}]
                    </td>
                </tr>[{captcha_tmpl}]
                <tr>
                    <td>
                        &nbsp;
                    </td>
                    <td>
                        {$mem_links}
                        [{cmt_submit}]
                        [{page_id}]
                        [{user_id}]
                        [{member_id}]
                        [{site_id}]
                    </td>
                </tr>
            </table>
FORMTEMP;

        $refresh = $lang->__('Refresh');
        $base_path = BASE_PATH;

        // Define the form template2 for image captcha.
        $captcha_tmpl = <<<CAPTCHATEMP

                <tr>
                    <td style="text-align: right; vertical-align: top;">
                        &nbsp;
                    </td>
                    <td>
                        <img src="{$base_path}/captcha" width="160" height="40" alt="" /><span style="float: right;">[{captcha_input_label}] [{captcha_input}] (<a style="font-size: 0.85em; font-weight: normal;" href="{$this->_action}">{$refresh}</a>)</span>
                        [{captcha}]
                    </td>
                </tr>

CAPTCHATEMP;

        $elems = array();

        // If CAPTCHA is turned on, set CAPTCHA form fields.
        if ($site->input_captcha == 'On') {

            // Create the 'captcha_input' form element.
            $captcha_input = new Moc10_Form_Element('text', 'captcha_input', $cap);
            $captcha_input->setLabel($lang->__('Enter the code:'));
            $captcha_input->setAttributes('size', '10');
            $captcha_input->setRequired(true);
            if ((!is_null($cap)) && (sha1(strtoupper($cap)) != $hash)) {
                $captcha_input->addValidator('Equal', true, 'Incorrect Captcha Code', $lang->__('The code is incorrect.'));
            }

            if (is_null($hash)) {
                $str = (string)Moc10_String::setString('')->random(7, true);
                $sess = Moc10_Session::getInstance();
                $sess->captcha = $str;
                $hash = sha1($str);
            }

            // Create the 'captcha' form element.
            $captcha = new Moc10_Form_Element('hidden', 'captcha', $hash);

            $tmpl = str_replace('[{captcha_tmpl}]', $captcha_tmpl, $tmpl);
            $elems = array($comment_author, $comment_content, $captcha_input, $captcha, $page_id, $user_id, $member_id, $site_id, $submit);

        } else {
            $tmpl = str_replace('[{captcha_tmpl}]', '', $tmpl);
            $elems = array($comment_author, $comment_content, $page_id, $user_id, $member_id, $site_id, $submit);
        }

        // Set the form template and add the form element objects to the form.
        $this->setTemplate($tmpl);
        $this->addElements($elems);

    }

}
