<?php
/**
 * Phire CMS
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.TXT.
 * It is also available through the world-wide-web at this URL:
 * http://www.phirecms.org/LICENSE.TXT
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@phirecms.org so we can send you a copy immediately.
 *
 * @category   Phire
 * @package    Phire_Form
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 */

/**
 * Phire_Form_Login
 *
 * @category   Phire
 * @package    Phire_Form
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 * @version    1.1.2
 */

class Phire_Form_Login extends Moc10_Form
{

    /**
     * Login error
     * @var string
     */
    public $error = null;

    /**
     * Form templates
     * @var array
     */
    protected $_templates = array('main' => null, 'mobile' => null);

    /**
     * $_POST fields
     * @var array
     */
    protected $_postFields = array('username' => null, 'password' => null, 'mobile' => null);

    /**
     * User object
     * @var Phire_Table_Users
     */
    protected $_user = null;

    /**
     * Config settings object
     * @var Phire_Table_Config
     */
    protected $_config = null;

    /**
     * IP addresses allowed
     * @var Array
     */
    protected $_ip_allowed = null;

    /**
     * IP addresses blocked
     * @var Array
     */
    protected $_ip_blocked = null;

    /**
     * Initialization method for the login form.
     *
     * @param  array $post
     * @return void
     */
    public function init($post = null)
    {

        $lang = new Phire_Language();

        $this->_config = new Phire_Table_Config();
        $this->_ip_allowed = ($this->_config->settings['ip_allowed'] != '') ? Moc10_String::setString($this->_config->settings['ip_allowed'])->split(',') : null;
        $this->_ip_blocked = ($this->_config->settings['ip_blocked'] != '') ? Moc10_String::setString($this->_config->settings['ip_blocked'])->split(',') : null;

        $this->_setFields($post);
        $this->_setTemplates();

        $this->setAttributes('id', 'loginForm');

        // Create the 'username' form element.
        $username = new Moc10_Form_Element('text', 'username', (string)Moc10_String::setString($this->_postFields['username'])->escape()->striptags());
        $username->setLabel($lang->__('Username:'));
        $username->setRequired(true);

        // Create the 'password' form element.
        $password = new Moc10_Form_Element('password', 'password', (string)Moc10_String::setString($this->_postFields['password'])->escape()->striptags());
        $password->setLabel($lang->__('Password:'));
        $password->setRequired(true);

        // Create the 'submit' form element.
        $submit = new Moc10_Form_Element('submit', 'submit', $lang->__('LOGIN'));
        $submit->setAttributes('class', 'submitBtn');
        $submit->setAttributes('style', 'float: right;');

        // Set whether it's a mobile form template or not.
        if (!is_null($this->_postFields['mobile'])) {
            $username->setAttributes('size', '27');
            $password->setAttributes('size', '27');
            $this->setTemplate($this->_templates['mobile']);
            $submit->setAttributes('class', 'submitBtn');
        } else {
            $username->setAttributes('style', 'width: 220px;');
            $password->setAttributes('style', 'width: 220px;');
            $username->setAttributes('size', '35');
            $password->setAttributes('size', '35');
            $this->setAttributes('onsubmit', "return loginFrm.validate();");
            $this->setTemplate($this->_templates['main']);
            $submit->setAttributes('class', 'submitBtn');
        }

        // Add the form element objects to the form.
        $this->addElements(array($username, $password, $submit));

    }

    /**
     * Method to authenticate the user login.
     *
     * @return void
     */
    public function auth()
    {

        $lang = new Phire_Language();

        $ip_filter = explode(',', $this->_config->settings['system_notification_filter']);
        $subnet = substr($_SERVER['REMOTE_ADDR'], 0, strrpos($_SERVER['REMOTE_ADDR'], '.')) . '.*';
        if (in_array($_SERVER['REMOTE_ADDR'], $ip_filter)) {
            $filtered = true;
        } else if (in_array($subnet, $ip_filter)) {
            $filtered = true;
        } else {
            $filtered = false;
        }

        // Attempt to retrieve the user.
        $this->_user = new Phire_Table_Users();
        $this->_user->findBy('username', $this->_postFields['username']);

        // Check the attempted login against the existence of the user.
        if (!isset($this->_user->user_id)) {

            $this->error = "        <div class=\"error\" style=\"padding: 20px 0 0 20px;\">" . $lang->__('The username could not be found.') . "</div>\n";
            if ((strpos($this->_config->settings['system_notification'], 'Attempted') !== false) && (!$filtered)) {
                $this->_notify('Attempted', $this->_postFields['username']);
            }

        // Check the number of failed login attempts.
        } else if (($this->_config->settings['failed_login_attempts'] > 0) && ($this->_user->failed_login_attempts >= $this->_config->settings['failed_login_attempts'])) {

            $this->_user->failed_login_attempts++;
            $this->_user->save();
            $this->error = "        <div class=\"error\" style=\"padding: 20px 0 0 20px;\">" . $lang->__('You have failed %1 or more times to login and are now blocked from the system. Contact the system administrator to reset your user account.', $this->_config->settings['failed_login_attempts']) . "</div>\n";
            if ((strpos($this->_config->settings['system_notification'], 'Failed') !== false) && (!$filtered)) {
                $this->_notify('Failed', $this->_postFields['username']);
            }

        // Check the attempted login against the user's correct password (password encoding off.)
        } else if (($this->_config->settings['user_password_encoding'] == 'Off') && ($this->_user->password != $this->_postFields['password'])) {

            $this->_user->failed_login_attempts++;
            $this->_user->save();
            $this->error = "        <div class=\"error\" style=\"padding: 20px 0 0 20px;\">" . $lang->__('The password was incorrect.') . "</div>\n";
            if ((strpos($this->_config->settings['system_notification'], 'Attempted') !== false) && (!$filtered)) {
                $this->_notify('Attempted', $this->_postFields['username']);
            }

        // Check the attempted login against the user's correct password (SHA1 password encoding on.)
        } else if (($this->_config->settings['user_password_encoding'] == 'SHA1') && ($this->_user->password != sha1($this->_postFields['password']))) {

            $this->_user->failed_login_attempts++;
            $this->_user->save();
            $this->error = "        <div class=\"error\" style=\"padding: 20px 0 0 20px;\">" . $lang->__('The password was incorrect.') . "</div>\n";
            if ((strpos($this->_config->settings['system_notification'], 'Attempted') !== false) && (!$filtered)) {
                $this->_notify('Attempted', $this->_postFields['username']);
            }

        // Check the attempted login against the user's correct password (MD5 password encoding on.)
        } else if (($this->_config->settings['user_password_encoding'] == 'MD5') && ($this->_user->password != md5($this->_postFields['password']))) {

            $this->_user->failed_login_attempts++;
            $this->_user->save();
            $this->error = "        <div class=\"error\" style=\"padding: 20px 0 0 20px;\">" . $lang->__('The password was incorrect.') . "</div>\n";
            if ((strpos($this->_config->settings['system_notification'], 'Attempted') !== false) && (!$filtered)) {
                $this->_notify('Attempted', $this->_postFields['username']);
            }

        // Check the attempted login against the user's allowed access.
        } else if ($this->_user->access == 'Blocked') {

            $this->_user->failed_login_attempts++;
            $this->_user->save();
            $this->error = "        <div class=\"error\" style=\"padding: 20px 0 0 20px;\">" . $lang->__('That user is currently blocked.') . "</div>\n";
            if ((strpos($this->_config->settings['system_notification'], 'Failed') !== false) && (!$filtered)) {
                $this->_notify('Failed', $this->_postFields['username']);
            }

        // Check the attempted login against whether or not the user has been assigned to any sites.
        } else if ($this->_user->allowed_sites == '') {

            $this->_user->failed_login_attempts++;
            $this->_user->save();
            $this->error = "        <div class=\"error\" style=\"padding: 20px 0 0 20px;\">" . $lang->__('That user is currently not assigned to any sites.') . "</div>\n";
            if ((strpos($this->_config->settings['system_notification'], 'Attempted') !== false) && (!$filtered)) {
                $this->_notify('Attempted', $this->_postFields['username']);
            }

        // Check the attempted login against whether or not the user has been verified.
        } else if (($this->_config->settings['user_verification'] == 'On') && ($this->_user->verified == 'No')) {

            $this->_user->failed_login_attempts++;
            $this->_user->save();
            $this->error = "        <div class=\"error\" style=\"padding: 20px 0 0 20px;\">" . $lang->__('That user has not been verified.') . "</div>\n";
            if ((strpos($this->_config->settings['system_notification'], 'Attempted') !== false) && (!$filtered)) {
                $this->_notify('Attempted', $this->_postFields['username']);
            }

        // Check the attempted login against the allowance of multiple sessions.
        } else if ($this->_config->settings['multiple_sessions_allowed'] == 'No') {

            $existing_user_sess = new Phire_Table_Sessions();
            $existing_user_sess->findAll(null, 'user_id', $this->_user->user_id);

            if (isset($existing_user_sess->rows[0])) {
                $this->_user->failed_login_attempts++;
                $this->_user->save();

                if (count($existing_user_sess->rows) > 1) {
                    $this->error = "        <div class=\"error\" style=\"padding: 20px 0 0 20px;\">" . $lang->__('That user is currently logged in from multiple locations and multiple sessions are not allowed.') . "</div>\n";
                } else {
                    $this->error = "        <div class=\"error\" style=\"padding: 20px 0 0 20px;\">" . $lang->__('That user is currently logged in from %1 and multiple sessions are not allowed.', $existing_user_sess->rows[0]['current_ip']) . "</div>\n";
                }
                if ((strpos($this->_config->settings['system_notification'], 'Attempted') !== false) && (!$filtered)) {
                    $this->_notify('Attempted', $this->_postFields['username']);
                }
            }

        // Check the attempted login against any IP screening filters.
        } else if ($this->_config->settings['ip_screening'] == 'On') {

            $brws = new Moc10_Session_Browser();

            if (isset($this->_ip_blocked[0]) && ($this->_ip_blocked[0] != '')) {
                if ((in_array($brws->subnet . '.*', $this->_ip_blocked)) || (in_array($brws->ip, $this->_ip_blocked))) {
                    $this->_user->failed_login_attempts++;
                    $this->_user->save();
                    $this->error = "        <div class=\"error\" style=\"padding: 20px 0 0 20px;\">" . $lang->__('The current IP address (%1) you are trying to log in from is blocked.', $brws->ip) . "</div>\n";
                    if ((strpos($this->_config->settings['system_notification'], 'Attempted') !== false) && (!$filtered)) {
                        $this->_notify('Attempted', $this->_postFields['username']);
                    }
                } else {
                    $this->error = null;
                }
            }

            if (isset($this->_ip_allowed[0]) && ($this->_ip_allowed[0] != '')) {
                if ((in_array($brws->subnet . '.*', $this->_ip_allowed)) || (in_array($brws->ip, $this->_ip_allowed))) {
                    $this->error = null;
                } else {
                    $this->_user->failed_login_attempts++;
                    $this->_user->save();
                    $this->error = "        <div class=\"error\" style=\"padding: 20px 0 0 20px;\">" . $lang->__('The current IP address (%1) you are trying to log in from is not allowed.', $brws->ip) . "</div>\n";
                    if ((strpos($this->_config->settings['system_notification'], 'Attempted') !== false) && (!$filtered)) {
                        $this->_notify('Attempted', $this->_postFields['username']);
                    }
                }
            }

        }

    }

    /**
     * Method to complete and save the user login.
     *
     * @return void
     */
    public function saveData()
    {

        $lang = new Phire_Language();

        $ip_filter = explode(',', $this->_config->settings['system_notification_filter']);
        $subnet = substr($_SERVER['REMOTE_ADDR'], 0, strrpos($_SERVER['REMOTE_ADDR'], '.')) . '.*';
        if (in_array($_SERVER['REMOTE_ADDR'], $ip_filter)) {
            $filtered = true;
        } else if (in_array($subnet, $ip_filter)) {
            $filtered = true;
        } else {
            $filtered = false;
        }

        // Session clean up.
        $ses = Moc10_Session::getInstance();
        if (isset($ses->expire_msg)) {
            unset($ses->expire_msg);
        }

        // Remove any lingering sessions in the DB table.
        $expired_sessions = new Phire_Table_Sessions();
        $expired_sessions->execute("SELECT * FROM " . DB_PREFIX . "sessions WHERE session_last_action < '" . date('Y-m-d H:i:s', (time() - 3600)) . "'");

        if (isset($expired_sessions->rows[0])) {
            foreach ($expired_sessions->rows as $exp_sess) {
                $exp = new Phire_Table_Sessions();
                $exp->findById($exp_sess->session_id);
                $exp->delete();
            }
        }

        // Create a browser and a mobile object.
        $brws = new Moc10_Session_Browser();
        $mob = new Moc10_Session_Mobile();

        if ($mob->isMobile !== false) {
            $brws_ua = $mob->ua;
            $login_from = 'Mobile';
        } else {
            $brws_ua = $brws->os . ', ' . $brws->browser . ' ' . $brws->version;
            $login_from = 'Desktop';
        }

        // Check for existing session based on user id, IP and user agent.
        $existing_session = new Phire_Table_Sessions();
        $existing_session->execute("SELECT * FROM " . DB_PREFIX . "sessions WHERE user_id = '" . $this->_user->user_id . "' AND current_ip = '" . $brws->ip . "' AND current_ua = '" . $brws_ua . "' LIMIT 1");

        // If a match is found, use the existing session.
        if (isset($existing_session->rows[0])) {

            $user_sess = new Phire_Table_Sessions();
            $user_sess->findById($existing_session->rows[0]['session_id']);
            $user_sess->session_start = date('Y-m-d H:i:s');
            $user_sess->session_last_action = date('Y-m-d H:i:s');

        // Else, start a new session.
        } else {

            $sess_fields = array('user_id' => $this->_user->user_id,
                                 'current_ip' => $brws->ip,
                                 'current_ua' => $brws_ua,
                                 'session_type' => $login_from,
                                 'session_start' => date('Y-m-d H:i:s'),
                                 'session_last_action' => date('Y-m-d H:i:s'));
            $user_sess = new Phire_Table_Sessions($sess_fields);

        }

        $user_sess->save();

        $last_login_form = ($this->_user->last_login_from == 'Mobile') ? "<span class=\"mobileIcon\"><span class=\"hide\">Mobile</span></span>" : '';

        // Set user's session details.
        $ses->user_id = $this->_user->user_id;
        $ses->user_sess_id = $user_sess->session_id;
        $ses->username = $this->_user->username;
        $ses->fname = $this->_user->fname;
        $ses->lname = $this->_user->lname;
        $ses->email = $this->_user->email;
        $ses->last_login = ($this->_user->last_login == '0000-00-00 00:00:00') ? $lang->__('Never') : date('l, M j, Y g:i A', strtotime($this->_user->last_login));
        $ses->last_login .= ($this->_user->last_ip == '') ? '' : '<br />(' . $last_login_form . $this->_user->last_ip . ')';
        $ses->access = $this->_user->access;
        $ses->sites = explode('|', $this->_user->allowed_sites);

        $open_auth = array();
        foreach ($ses->sites as $sid) {
            $site = new Phire_Table_Sites();
            $site->findbyId($sid);
            $open_auth[$sid] = $site->open_authoring;
        }

        $ses->open_authoring = $open_auth;

        // Save the user login details.
        $this->_user->last_login = date('Y-m-d H:i:s');
        $this->_user->last_login_from = $login_from;
        $this->_user->last_ip = $_SERVER['REMOTE_ADDR'];
        $this->_user->failed_login_attempts = 0;
        $this->_user->save();

        if ((strpos($this->_config->settings['system_notification'], 'Successful') !== false) && (!$filtered)) {
            $this->_notify('Successful', $this->_postFields['username']);
        }

        if ($this->_config->settings['table_optimization'] == 'On') {
            $periods = array('Monthly' => 2592000,
                             'Weekly' => 604800,
                             'Daily' => 86400);
            if ($this->_config->settings['last_optimization'] == '0000-00-00 00:00:00') {
                $this->_optimize();
            } else {
                if ((time() - strtotime($this->_config->settings['last_optimization'])) > $periods[$this->_config->settings['optimization_period']]) {
                    $this->_optimize();
                }
            }
        }

    }

    /**
     * Method to set the post fields.
     *
     * @param  array $post
     * @return void
     */
    protected function _setFields($post = null)
    {

        if (!is_null($post)) {
            foreach ($post as $key => $value) {
                if (array_key_exists($key, $this->_postFields)) {
                    $this->_postFields[$key] = (isset($post[$key])) ? $post[$key] : null;
                }
            }
        }

    }

    /**
     * Method to set the form templates.
     *
     * @return void
     */
    protected function _setTemplates()
    {

        $lang = new Phire_Language();
        $please = $lang->__('Please Login');
        $forgot = $lang->__('Forgot?');

        $sys_path = BASE_PATH . SYS_PATH;

        // Define the main form template.
        $this->_templates['main'] = <<<FORMTEMP
            <table id="loginTable" cellspacing="0" cellpadding="0">
                <tr>
                    <td colspan="2">
                        <h2 class="blueHeader" style="margin-left: 0;">{$please}</h2>
                    </td>
                </tr>
                <tr>
                    <td class="textTopRight">
                        [{username_label}]
                    </td>
                    <td>
                        [{username}]
                    </td>
                </tr>
                <tr>
                    <td class="textTopRight">
                        [{password_label}]
                    </td>
                    <td>
                        [{password}]
                    </td>
                </tr>
                <tr>
                    <td>
                        &nbsp;
                    </td>
                    <td>
                        [{submit}]
                        <a class="smallLink" href="{$sys_path}/forgot.php">{$forgot}</a>
                    </td>
                </tr>
            </table>
            <script type="text/javascript">
                loginFormInit();
                loginFrm.fo.username.focus();
            </script>
FORMTEMP;

        $mobile = (!is_null($this->_postFields['mobile'])) ? "<input type=\"hidden\" name=\"mobile\" value=\"true\" />" : null;

        // Define the mobile form template.
        $this->_templates['mobile'] = <<<FORMTEMP
            <table id="loginTable" cellspacing="0" cellpadding="0">
                <tr>
                    <td colspan="2">
                        <h2 class="blueHeader" style="margin-left: 0;">{$please}</h2>
                    </td>
                </tr>
                <tr>
                    <td class="textTopRight" style="width: 40%;">
                        [{username_label}]
                    </td>
                    <td>
                        [{username}]
                    </td>
                </tr>
                <tr>
                    <td class="textTopRight" style="width: 60%;">
                        [{password_label}]
                    </td>
                    <td>
                        [{password}]
                    </td>
                </tr>
                <tr>
                    <td>
                        &nbsp;
                    </td>
                    <td>
                        [{submit}] {$mobile}
                        <a class="smallLink" href="{$sys_path}/mobile/forgot.php">{$forgot}</a>
                    </td>
                </tr>
            </table>
FORMTEMP;

    }

    /**
     * Method to optimize the system database tables.
     *
     * @return void
     */
    protected function _optimize()
    {

        $db = Moc10_Db::getInstance();
        $tables = Moc10_Db::getInstance()->interface->getTables();
        foreach ($tables as $table) {
            $db->interface->query("OPTIMIZE TABLE " . $table);
        }
        $config = new Phire_Table_Config();
        $config->findById('last_optimization');
        $config->value = date('Y-m-d H:i:s');
        $config->save();

    }

    /**
     * Method to notify the system admins of attempted logins.
     *
     * @param  string $type
     * @param  string $user
     * @return void
     */
    protected function _notify($type, $user)
    {

        $lang = new Phire_Language();

        $admins = new Phire_Table_Users();
        $admins->findAll('access', 'Admin');

        if (isset($admins->rows[0])) {

            $rcpts = array();

            foreach ($admins->rows as $admin) {
                $rcpts[] = array('name' => $admin->fname . ' ' . $admin->lname, 'email' => $admin->email);
            }

            $noreply = 'noreply@' . ((stripos($_SERVER['HTTP_HOST'], 'www.') !== false) ? str_replace('www.', '', $_SERVER['HTTP_HOST']) : $_SERVER['HTTP_HOST']);

            switch ($type) {
                case 'Attempted':
                    $subject = $lang->__('Phire CMS: Attempted Login (Username: %1)', $user);
                    $message = "[{name}],\n\n" . $lang->__('There has been an attempted login to the Phire CMS admin area at %1 from IP address %2 using the username \'%3\'.', array('http://' . $_SERVER['HTTP_HOST'] . '/', $_SERVER['REMOTE_ADDR'], $user)) . "\n\n" . $lang->__('Thank You,') . "\n\n" . $lang->__('The Phire CMS Team') . "\n\n";
                    break;

                case 'Failed':
                    $subject = $lang->__('Phire CMS: Failed/Blocked Login (Username: %1)', $user);
                    $message = "[{name}],\n\n" . $lang->__('There has been an attempted login to the Phire CMS admin area at %1 from IP address %2 using the username \'%3\' that has failed and has blocked that user from the system.', array('http://' . $_SERVER['HTTP_HOST'] . '/', $_SERVER['REMOTE_ADDR'], $user)) . "\n\n" . $lang->__('Thank You,') . "\n\n" . $lang->__('The Phire CMS Team') . "\n\n";
                    break;

                case 'Successful':
                    $subject = $lang->__('Phire CMS: Successful Login (Username: %1)', $user);
                    $message = "[{name}],\n\n" . $lang->__('There has been a successful login attempt to the Phire CMS admin area at %1 from IP address %2 using the username \'%3\'.', array('http://' . $_SERVER['HTTP_HOST'] . '/', $_SERVER['REMOTE_ADDR'], $user)) . "\n\n" . $lang->__('Thank You,') . "\n\n" . $lang->__('The Phire CMS Team') . "\n\n";
                    break;
            }

            $mail = new Moc10_Mail($rcpts, null, $subject);
            $mail->setHeaders(array('From' => array($noreply, $noreply),
                                    'Reply-To' => array($noreply, $noreply)));

            $mail->setText($message);
            $mail->init();
            $mail->send();

        }

    }

}
