<?php
/**
 * Phire CMS
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.TXT.
 * It is also available through the world-wide-web at this URL:
 * http://www.phirecms.org/LICENSE.TXT
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@phirecms.org so we can send you a copy immediately.
 *
 * @category   Phire
 * @package    Phire_Plugins
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 */

/**
 * Phire_Plugins
 *
 * @category   Phire
 * @package    Phire_Plugins
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 * @version    1.1.2
 */

class Phire_Plugins
{

    /**
     * Content to process
     * @var string
     */
    protected $_content = null;

    /**
     * URL request
     * @var string
     */
    protected $_url = null;

    /**
     * Site ID
     * @var string
     */
    protected $_site_id = null;

    /**
     * Page 404 flag
     * @var boolean
     */
    protected $_is404 = false;

    /**
     * Page/Section ID
     * @var string|int
     */
    protected $_id = null;

    /**
     * System root
     * @var string
     */
    protected $_sysdocroot = null;

    /**
     * Constructor
     *
     * Instantiate the plugin object.
     *
     * @param  string $content
     * @param  string $url
     * @param  string $site_id
     * @param  boolean $is404
     * @param  string|int $id
     * @return void
     */
    public function __construct($content, $url, $site_id, $is404, $id = null)
    {

        $cfg = new Phire_Table_Config();
        $this->_sysdocroot = $cfg->settings['system_docroot'];

        $this->_content = $content;
        $this->_url = $url;
        $this->_site_id = $site_id;
        $this->_is404 = $is404;
        $this->_id = $id;

        if (file_exists($this->_sysdocroot)) {

            $pluginDir = new Moc10_File_Dir($this->_sysdocroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/');

            if ((count($pluginDir->files) > 1) || (isset($pluginDir->files[0]) && ($pluginDir->files[0] != 'index.html'))) {

                $plugins = new Phire_Table_Plugins();
                $plugins->findAll();

                // Iterate through any plugins and include any plugin controllers.
                if (isset($plugins->rows[0])) {
                    foreach ($plugins->rows as $plugin) {
                        if ($plugin->plugin_controller != '') {
                            include_once $this->_sysdocroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/' . $plugin->plugin_folder . '/' . $plugin->plugin_controller;
                        }
                    }
                }

            }

        }

    }

    /**
     * Static method to instantiate the plugin object and
     * return itself to facilitate chaining methods together.
     *
     * @param  string $cont
     * @param  string $u
     * @param  string|int $sid
     * @param  boolean $is4
     * @param  string|int $pid
     * @return Phire_Plugins
     */
    public static function init($cont, $u, $sid, $is4, $pid = null)
    {

        return new self($cont, $u, $sid, $is4, $pid);

    }


    /**
     * Get content within the content property.
     *
     * @return string
     */
    public function getContent()
    {

        return $this->_content;

    }

}
